@@ -1,24 +1,24 @@
 /*****************************************************************************\
- *  $Id: dshgroup.c 1009 2006-02-14 16:11:43Z grondo $
+ *  $Id$
  *****************************************************************************
  *  Copyright (C) 2005-2006 The Regents of the University of California.
  *  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
  *  Written by Mark Grondona <mgrondona@llnl.gov>
  *  UCRL-CODE-2003-005.
- *  
+ *
  *  This file is part of Pdsh, a parallel remote shell program.
  *  For details, see <http://www.llnl.gov/linux/pdsh/>.
- *  
+ *
  *  Pdsh is free software; you can redistribute it and/or modify it under
  *  the terms of the GNU General Public License as published by the Free
  *  Software Foundation; either version 2 of the License, or (at your option)
  *  any later version.
- *  
+ *
  *  Pdsh is distributed in the hope that it will be useful, but WITHOUT ANY
  *  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  *  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
  *  details.
- *  
+ *
  *  You should have received a copy of the GNU General Public License along
  *  with Pdsh; if not, write to the Free Software Foundation, Inc.,
  *  59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
@@ -28,6 +28,10 @@
 #  include "config.h"
 #endif
 
+#ifndef DSHGROUP_PATH
+#define DSHGROUP_PATH "/etc/dsh/group"
+#endif
+
 #include <unistd.h> /* access */
 #include <stdlib.h> /* getenv */
 #include <string.h>
@@ -43,7 +47,7 @@
 #if STATIC_MODULES
 #  define pdsh_module_info dshgroup_module_info
 #  define pdsh_module_priority dshgroup_module_priority
-#endif    
+#endif
 
 int pdsh_module_priority = DEFAULT_MODULE_PRIORITY;
 
@@ -51,22 +55,20 @@ static hostlist_t read_groupfile(opt_t *opt);
 static int dshgroup_postop (opt_t *);
 static int dshgroup_process_opt(opt_t *, int, char *);
 
-static List _grouplist_append (List l, char *str);
-
 static List groups = NULL;
 static List exgroups = NULL;
 
-/* 
+/*
  * Export pdsh module operations structure
  */
 struct pdsh_module_operations dshgroup_module_ops = {
-    (ModInitF)       NULL, 
-    (ModExitF)       NULL, 
-    (ModReadWcollF)  read_groupfile, 
+    (ModInitF)       NULL,
+    (ModExitF)       NULL,
+    (ModReadWcollF)  read_groupfile,
     (ModPostOpF)     dshgroup_postop,
 };
 
-/* 
+/*
  * Export rcmd module operations
  */
 struct pdsh_rcmd_operations dshgroup_rcmd_ops = {
@@ -75,26 +77,26 @@ struct pdsh_rcmd_operations dshgroup_rcmd_ops = {
     (RcmdF)      NULL,
 };
 
-/* 
+/*
  * Export module options
  */
-struct pdsh_module_option dshgroup_module_options[] = 
- { { 'g', "groupname", "target hosts in dsh group \"groupname\"", 
+struct pdsh_module_option dshgroup_module_options[] =
+ { { 'g', "groupname", "target hosts in dsh group \"groupname\"",
    DSH | PCP, (optFunc) dshgroup_process_opt },
-   { 'X', "groupname", "exclude hosts in dsh group \"groupname\"", 
+   { 'X', "groupname", "exclude hosts in dsh group \"groupname\"",
    DSH | PCP, (optFunc) dshgroup_process_opt },
    PDSH_OPT_TABLE_END
  };
 
-/* 
- * Machines module info 
+/*
+ * Machines module info
  */
 struct pdsh_module pdsh_module_info = {
   "misc",
   "dshgroup",
   "Mark Grondona <mgrondona@llnl.gov>",
   "Read list of targets from dsh-style \"group\" files",
-  DSH | PCP, 
+  DSH | PCP,
   &dshgroup_module_ops,
   &dshgroup_rcmd_ops,
   &dshgroup_module_options[0],
@@ -104,10 +106,10 @@ static int dshgroup_process_opt(opt_t *pdsh_opt, int opt, char *arg)
 {
     switch (opt) {
     case 'g':
-        groups = _grouplist_append (groups, arg);
+        groups = list_split_append (groups, ",", arg);
         break;
     case 'X':
-        exgroups = _grouplist_append (exgroups, arg);
+        exgroups = list_split_append (exgroups, ",", arg);
         break;
     default:
         err ("%p: dshgroup_process_opt: invalid option `%c'\n", opt);
@@ -119,30 +121,29 @@ static int dshgroup_process_opt(opt_t *pdsh_opt, int opt, char *arg)
 
 static hostlist_t _read_groupfile (const char *group)
 {
-    char groupfile[4096] = "";
-    char backupfile[4096] = "";
+    int maxpathlen;
+    char path [4096];
     char *home = getenv("HOME");
+    char *dshgroup_path = getenv("DSHGROUP_PATH");
 
+    maxpathlen = sizeof (path) - 1;
 
-    snprintf (backupfile, 4096, "/etc/dsh/group/%s", group);
+    if (!dshgroup_path)
+        dshgroup_path = DSHGROUP_PATH;
 
     if (home) {
-        snprintf (groupfile, 4096, "%s/.dsh/group/%s", home, group);
-        if (access (groupfile, R_OK) >= 0) 
-            return read_wcoll(groupfile, NULL);
-    } else
+        int n;
+        n = snprintf (path, maxpathlen,"%s/.dsh/group:%s", home, dshgroup_path);
+        if (n <= 0 || n > maxpathlen)
+            errx ("%p: dshgroup: search path (%s/.dsh/group:%s) overflow\n",
+                    home, dshgroup_path);
+    }
+    else {
         err ("%p: dshgroup: warning: Unable to read $HOME env var\n");
+        strncpy (path, dshgroup_path, sizeof (path));
+    }
 
-    if (access (backupfile, R_OK) >= 0)
-        return read_wcoll(backupfile, NULL);
-
-    err ("%p: unable to read group file from ");
-    if (home)
-        err ("%s or %s\n", groupfile, backupfile);
-    else
-        err ("%s\n", backupfile);
-
-    return NULL;
+    return read_wcoll_path (path, group);
 }
 
 static hostlist_t _read_groups (List grouplist)
@@ -158,7 +159,7 @@ static hostlist_t _read_groups (List grouplist)
 
         if (l == NULL)
             continue;
-    
+
         if (hl == NULL) {
             hl = l;
         } else {
@@ -204,7 +205,7 @@ _delete_all (hostlist_t hl, hostlist_t dl)
 static int dshgroup_postop (opt_t *opt)
 {
     hostlist_t hl = NULL;
-    
+
     if (!opt->wcoll || !exgroups)
         return (0);
 
@@ -216,23 +217,6 @@ static int dshgroup_postop (opt_t *opt)
     return 0;
 }
 
-static List _grouplist_append (List l, char *str)
-{
-    List tmp = list_split (",", str);
-    ListIterator i = NULL;
-    char *attr = NULL;
-
-    if (l == NULL) 
-        return ((l = tmp));
-
-    i = list_iterator_create (tmp);
-    while ((attr = list_next (i))) 
-        list_append (l, Strdup(attr));
-    list_destroy (tmp);
-
-    return (l);
-}
-
 /*
  * vi: tabstop=4 shiftwidth=4 expandtab
  */
