@@ -1,24 +1,24 @@
 /*****************************************************************************\
- *  $Id: genders.c 1155 2008-03-11 21:40:35Z grondo $
+ *  $Id$
  *****************************************************************************
  *  Copyright (C) 2001-2006 The Regents of the University of California.
  *  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
  *  Written by Jim Garlick <garlick@llnl.gov>.
  *  UCRL-CODE-2003-005.
- *  
+ *
  *  This file is part of Pdsh, a parallel remote shell program.
  *  For details, see <http://www.llnl.gov/linux/pdsh/>.
- *  
+ *
  *  Pdsh is free software; you can redistribute it and/or modify it under
  *  the terms of the GNU General Public License as published by the Free
  *  Software Foundation; either version 2 of the License, or (at your option)
  *  any later version.
- *  
+ *
  *  Pdsh is distributed in the hope that it will be useful, but WITHOUT ANY
  *  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  *  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
  *  details.
- *  
+ *
  *  You should have received a copy of the GNU General Public License along
  *  with Pdsh; if not, write to the Free Software Foundation, Inc.,
  *  59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
@@ -37,7 +37,7 @@
 #include "src/common/err.h"
 #include "src/common/xmalloc.h"
 #include "src/common/split.h"
-#include "src/pdsh/ltdl.h"
+#include "src/common/xstring.h"
 #include "src/pdsh/mod.h"
 #include "src/pdsh/rcmd.h"
 
@@ -50,7 +50,7 @@
 #if STATIC_MODULES
 #  define pdsh_module_info genders_module_info
 #  define pdsh_module_priority genders_module_priority
-#endif    
+#endif
 
 int pdsh_module_priority = DEFAULT_MODULE_PRIORITY;
 
@@ -70,27 +70,25 @@ static int        genders_postop(opt_t *);
 static bool allnodes   = false;
 static bool opt_i      = false;
 #endif /* !GENDERS_G_ONLY */
+static bool genders_opt_invoked = false;
+static bool generate_altnames =   false;
 
 static genders_t gh    = NULL;
 static char *gfile     = NULL;
 static List attrlist   = NULL;
 static List excllist   = NULL;
 
-static lt_dlhandle dlh = NULL;
-static lt_ptr g_query_addr = NULL;
-typedef int (*g_query)(genders_t, char **, int, char *);
-
-/* 
+/*
  * Export pdsh module operations structure
  */
 struct pdsh_module_operations genders_module_ops = {
-    (ModInitF)       genders_init, 
-    (ModExitF)       genders_fini, 
-    (ModReadWcollF)  genders_wcoll, 
+    (ModInitF)       genders_init,
+    (ModExitF)       genders_fini,
+    (ModReadWcollF)  genders_wcoll,
     (ModPostOpF)     genders_postop,
 };
 
-/* 
+/*
  * Export rcmd module operations
  */
 struct pdsh_rcmd_operations genders_rcmd_ops = {
@@ -99,16 +97,16 @@ struct pdsh_rcmd_operations genders_rcmd_ops = {
     (RcmdF)      NULL,
 };
 
-/* 
+/*
  * Export module options
  */
-struct pdsh_module_option genders_module_options[] = 
- { 
-   { 'g', "query,...", 
+struct pdsh_module_option genders_module_options[] =
+ {
+   { 'g', "query,...",
      "target nodes using genders query",
-     DSH | PCP, (optFunc) genders_process_opt 
+     DSH | PCP, (optFunc) genders_process_opt
    },
-   { 'X', "query,...", 
+   { 'X', "query,...",
      "exclude nodes using genders query",
      DSH | PCP, (optFunc) genders_process_opt
    },
@@ -119,8 +117,8 @@ struct pdsh_module_option genders_module_options[] =
    { 'i', NULL, "request alternate or canonical hostnames if applicable",
      DSH | PCP, (optFunc) genders_process_opt
    },
-   { 'a', NULL, "target all nodes except those with \"pdsh_all_skip\" attribute", 
-     DSH | PCP, (optFunc) genders_process_opt 
+   { 'a', NULL, "target all nodes except those with \"pdsh_all_skip\" attribute",
+     DSH | PCP, (optFunc) genders_process_opt
    },
    { 'A', NULL, "target all nodes listed in genders database",
      DSH | PCP, (optFunc) genders_process_opt
@@ -129,8 +127,8 @@ struct pdsh_module_option genders_module_options[] =
    PDSH_OPT_TABLE_END
  };
 
-/* 
- * Genders module info 
+/*
+ * Genders module info
  */
 struct pdsh_module pdsh_module_info = {
     "misc",
@@ -141,7 +139,7 @@ struct pdsh_module pdsh_module_info = {
 #endif /* GENDERS_G_ONLY */
     "Jim Garlick <garlick@llnl.gov>",
     "target nodes using libgenders and genders attributes",
-    DSH | PCP, 
+    DSH | PCP,
 
     &genders_module_ops,
     &genders_rcmd_ops,
@@ -154,6 +152,7 @@ struct pdsh_module pdsh_module_info = {
 static genders_t  _handle_create();
 static hostlist_t _genders_to_altnames(genders_t g, hostlist_t hl);
 static hostlist_t _read_genders(List l);
+static hostlist_t _read_genders_attr(char *query);
 static void       _genders_opt_verify(opt_t *opt);
 static int        _delete_all (hostlist_t hl, hostlist_t dl);
 static int        register_genders_rcmd_types (opt_t *opt);
@@ -167,8 +166,8 @@ genders_process_opt(opt_t *pdsh_opts, int opt, char *arg)
 {
     switch (opt) {
 #if !GENDERS_G_ONLY
-    case 'a':  
-        /* For -a, exclude nodes with "pdsh_all_skip" */ 
+    case 'a':
+        /* For -a, exclude nodes with "pdsh_all_skip" */
         excllist = list_split_append (excllist, ",", "pdsh_all_skip");
     case 'A':
         allnodes = true;
@@ -191,17 +190,13 @@ genders_process_opt(opt_t *pdsh_opts, int opt, char *arg)
         return -1;
         break;
     }
+    genders_opt_invoked = true;
     return 0;
 }
 
 static int
 genders_init(void)
 {
-    if (!(dlh = lt_dlopen(NULL)))
-        errx("%p: Error loading self: %s\n", lt_dlerror());
-
-    g_query_addr = lt_dlsym(dlh, "genders_query");
-
     return 0;
 }
 
@@ -217,15 +212,17 @@ genders_fini(void)
     if ((gh != NULL) && (genders_handle_destroy(gh) < 0))
         errx("%p: Error destroying genders handle: %s\n", genders_errormsg(gh));
 
-    lt_dlclose(dlh);
     return 0;
 }
 
-static hostlist_t 
+static hostlist_t
 genders_wcoll(opt_t *opt)
 {
     _genders_opt_verify(opt);
 
+    if (opt->wcoll)
+        return (NULL);
+
 #if GENDERS_G_ONLY
     if (!attrlist)
         return NULL;
@@ -237,35 +234,86 @@ genders_wcoll(opt_t *opt)
     if (gh == NULL)
         gh = _handle_create();
 
+    generate_altnames = true;
     return _read_genders(attrlist);
 }
 
+static hostlist_t hostlist_intersect (hostlist_t h1, hostlist_t h2)
+{
+    char *host;
+    hostlist_t r = hostlist_create (NULL);
+    hostlist_iterator_t i = hostlist_iterator_create (h1);
+
+    while ((host = hostlist_next (i))) {
+        if (hostlist_find (h2, host) >= 0) {
+            hostlist_push_host (r, host);
+	}
+        free (host);
+    }
+    hostlist_iterator_destroy (i);
+    return (r);
+}
+
+static hostlist_t genders_query_with_altnames (char *query)
+{
+    hostlist_t r = _read_genders_attr (query);
+    hostlist_t altlist = _genders_to_altnames (gh, r);
+    hostlist_push_list (r, altlist);
+    hostlist_destroy (altlist);
+    return (r);
+}
+
+/*
+ *  Filter hostlist hl on a list of genders queries in query_list.
+ *   Multiple queries are ORed together, so a given host must only
+ *   match a single query.
+ */
+static hostlist_t genders_filter (hostlist_t hl, List query_list)
+{
+    char *s;
+    ListIterator i;
+    hostlist_t result;
+
+    if ((query_list == NULL) || (list_count (query_list) == 0))
+        return hl;
+
+    if ((i = list_iterator_create (query_list)) == NULL) {
+        err ("%p: genders: failed to create list or hostlist iterator\n");
+        return hl;
+    }
+
+    /*
+     *  Result is the union of the intersection of each genders query
+     *   with the incoming hostlist [hl]
+     */
+    result = hostlist_create (NULL);
+    while ((s = list_next (i))) {
+        hostlist_t ghl = genders_query_with_altnames (s);
+        hostlist_t r = hostlist_intersect (hl, ghl);
+        hostlist_destroy (ghl);
+
+        hostlist_push_list (result, r);
+    }
+    list_iterator_destroy (i);
+    hostlist_uniq (result);
+    hostlist_destroy (hl);
+    return (result);
+}
+
 static int
 genders_postop(opt_t *opt)
 {
     hostlist_t hl = NULL;
-    bool altnames = false;
 
     if (!opt->wcoll)
         return (0);
 
-    /*
-     *  Grab altnames if gend_attr or allnodes given and !opt_i,
-     *   or if opt_i and neither gend_attr or allnodes given.
-     */
-    altnames = (opt_i && !(attrlist || allnodes)) 
-            || (!opt_i && (attrlist || allnodes));
-
-    /*
-     *  Return early if no genders options specified.
-     *   This way genders file is not opened unless necessary.
-     */
-    if (!attrlist && !allnodes && !excllist && !opt_i)
-        return (0);
-
     if (gh == NULL)
         gh = _handle_create();
 
+    if (attrlist)
+        opt->wcoll = genders_filter (opt->wcoll, attrlist);
+
     if (excllist && (hl = _read_genders (excllist))) {
         hostlist_t altlist = _genders_to_altnames (gh, hl);
         _delete_all (opt->wcoll, hl);
@@ -276,20 +324,29 @@ genders_postop(opt_t *opt)
     }
 
 #if !GENDERS_G_ONLY
-    if (altnames) {
+    /*
+     *  Genders module returns altnames by default, but only
+     *   when genders fills in wcoll or with -i, not when filtering via
+     *   -g or -X.
+     */
+    if ((generate_altnames && !opt_i) || (!generate_altnames && opt_i)) {
         hostlist_t hl = opt->wcoll;
         opt->wcoll = _genders_to_altnames(gh, hl);
         hostlist_destroy(hl);
     }
 #endif
 
+    /*
+     *  Apply any pdsh_rcmd_type setting from genders file
+     *   based on current opt->wcoll.
+     */
     register_genders_rcmd_types (opt);
 
     return (0);
 }
 
 
-/* 
+/*
  * Verify options passed to this module
  */
 static void
@@ -302,18 +359,8 @@ _genders_opt_verify(opt_t *opt)
  *   }
  */
     if (allnodes && (attrlist != NULL))
-        errx("%p: Do not specify -a with -g\n");
+        errx("%p: Do not specify -A or -a with -g\n");
 #endif /* !GENDERS_G_ONLY */
-
-    if(opt->wcoll) {
-#if !GENDERS_G_ONLY
-        if (allnodes)
-            errx("%p: Do not specify -a with other node selection options\n");
-#endif /* !GENDERS_G_ONLY */
-        if (attrlist)
-            errx("%p: Do not specify -g with other node selection options\n");
-    }
-
     return;
 }
 
@@ -359,7 +406,7 @@ _genders_to_altnames(genders_t g, hostlist_t hl)
          *  If node not found, attempt to lookup canonical name via
          *   altername name.
          */
-        if ((rc < 0) && (genders_errnum(g) == GENDERS_ERR_NOTFOUND)) 
+        if ((rc < 0) && (genders_errnum(g) == GENDERS_ERR_NOTFOUND))
             rc = genders_getnodes (g, &altname, 1, altattr, host);
 
         if (hostlist_push_host(retlist, (rc > 0 ? altname : host)) <= 0)
@@ -375,7 +422,7 @@ _genders_to_altnames(genders_t g, hostlist_t hl)
     return (retlist);
 }
 
-static hostlist_t 
+static hostlist_t
 _genders_to_hostlist(genders_t gh, char **nodes, int nnodes)
 {
     hostlist_t hl = NULL;
@@ -394,16 +441,51 @@ _genders_to_hostlist(genders_t gh, char **nodes, int nnodes)
     return hl;
 }
 
+static char * genders_filename_create (char *file)
+{
+    char *genders_file;
+    const char *gdir = getenv ("PDSH_GENDERS_DIR");
+
+    /*
+     *  Return a copy of filename if user specified an
+     *   absolute or relative path:
+     */
+    if (file[0] == '/' || file[0] == '.')
+        return Strdup (file);
+
+    /*
+     *  Otherwise, append filename to
+     *   PDSH_GENDERS_DIR (or /etc by default)
+     */
+    genders_file = gdir ? Strdup (gdir) : Strdup ("/etc");
+    xstrcatchar (&genders_file, '/');
+    xstrcat (&genders_file, file);
+
+    return (genders_file);
+}
+
 static genders_t _handle_create()
 {
+    char *gfile_env;
+    char *genders_file = NULL;
     genders_t gh = NULL;
 
     if ((gh = genders_handle_create()) == NULL)
         errx("%p: Unable to create genders handle: %m\n");
 
-    /* assumes genders file in default location */
-    if (genders_load_data(gh, gfile) < 0)
-        errx("%p: Unable to open genders file: %s\n", genders_errormsg(gh));
+    if (gfile)
+        genders_file = genders_filename_create (gfile);
+    else if ((gfile_env = getenv ("PDSH_GENDERS_FILE")))
+        genders_file = genders_filename_create (gfile_env);
+    else
+        genders_file = genders_filename_create ("genders");
+
+    /*
+     *  Only exit on error from genders_load_data() if an genders
+     *   module option was explicitly invoked:
+     */
+    if ((genders_load_data(gh, genders_file) < 0) && genders_opt_invoked)
+        errx("%p: %s: %s\n", genders_file, genders_errormsg(gh));
 
     return gh;
 }
@@ -415,6 +497,7 @@ static genders_t _handle_create()
  *
  *  Returns NULL if no '=' found.
  */
+#if !HAVE_GENDERS_QUERY
 static char *
 _get_val(char *attr)
 {
@@ -430,8 +513,9 @@ _get_val(char *attr)
 
     return (val);
 }
+#endif /* !HAVE_GENDERS_QUERY */
 
-static hostlist_t 
+static hostlist_t
 _read_genders_attr(char *query)
 {
     hostlist_t hl = NULL;
@@ -441,21 +525,22 @@ _read_genders_attr(char *query)
     if ((len = genders_nodelist_create(gh, &nodes)) < 0)
         errx("%p: genders: nodelist_create: %s\n", genders_errormsg(gh));
 
-    if (g_query_addr) {
-        if ((nnodes = ((g_query)g_query_addr)(gh, nodes, len, query)) < 0) {
-            errx("%p: Error querying genders for query \"%s\": %s\n", 
-                 query ?: "(all)", genders_errormsg(gh));
-        }
+#if HAVE_GENDERS_QUERY
+    if ((nnodes = genders_query (gh, nodes, len, query)) < 0) {
+        errx("%p: Error querying genders for query \"%s\": %s\n",
+                query ? query : "(all)", genders_errormsg(gh));
     }
-    else {
+#else /* !HAVE_GENDERS_QUERY */
+    {
         /* query defaults to just an attribute or attribute=value pair */
         char *val;
         val = _get_val(query);
         if ((nnodes = genders_getnodes(gh, nodes, len, query, val)) < 0) {
-            errx("%p: Error querying genders for attr \"%s\": %s\n", 
-                 query ?: "(all)", genders_errormsg(gh));
+            errx("%p: Error querying genders for attr \"%s\": %s\n",
+                 query ? query : "(all)", genders_errormsg(gh));
         }
     }
+#endif /* HAVE_GENDERS_QUERY */
 
     hl = _genders_to_hostlist(gh, nodes, nnodes);
 
@@ -467,10 +552,9 @@ _read_genders_attr(char *query)
     return hl;
 }
 
-static hostlist_t 
+static hostlist_t
 _read_genders (List attrs)
 {
-    ListIterator i  = NULL;
     hostlist_t   hl = NULL;
     char *    query = NULL;
 
@@ -480,10 +564,7 @@ _read_genders (List attrs)
     if ((attrs == NULL) || (list_count (attrs) == 0))
         return NULL;
 
-    if ((i = list_iterator_create (attrs)) == NULL)
-        errx ("genders: unable to create list iterator: %m\n");
-
-    while ((query = list_next (i))) {
+    while ((query = list_pop (attrs))) {
         hostlist_t l = _read_genders_attr (query);
 
         if (hl == NULL) {
@@ -492,19 +573,18 @@ _read_genders (List attrs)
             hostlist_push_list (hl, l);
             hostlist_destroy (l);
         }
+        Free ((void **)&query);
     }
 
-    list_iterator_destroy (i);
-
     hostlist_uniq (hl);
 
     return (hl);
 }
 
-static int 
+static int
 attrval_by_altname (genders_t g, const char *host, const char *attr,
                            char *val, int len)
-{   
+{
     char *altname = NULL;
     char *altattr = GENDERS_ALTNAME_ATTRIBUTE;
     int maxlen = _maxnamelen (g);
@@ -525,8 +605,8 @@ attrval_by_altname (genders_t g, const char *host, const char *attr,
  *  Parse the value of "pdsh_rcmd_type" and split into user and rcmd
  *   strings, passing rcmd name (if any) in *rp, and user name (if any)
  *   in *up.
- *   
- *  Allows pdsh_rcmd_type to be set to [user@][rcmd], where user@ and 
+ *
+ *  Allows pdsh_rcmd_type to be set to [user@][rcmd], where user@ and
  *   rcmd are both optional. (i.e. you can set user or rcmd or both)
  */
 static int rcmd_type_parse (char *val, char **rp, char **up)
@@ -556,10 +636,10 @@ register_genders_rcmd_types (opt_t *opt)
     char rcmd_attr[] = "pdsh_rcmd_type";
     hostlist_iterator_t i = NULL;
 
-    if (!opt->wcoll) 
+    if (!opt->wcoll)
         return (0);
 
-    /* 
+    /*
      *  Assume no nodes have "pdsh_rcmd_type" attr if index fails:
      */
     if (genders_index_attrvals (gh, rcmd_attr) < 0)
@@ -575,23 +655,23 @@ register_genders_rcmd_types (opt_t *opt)
          *  If host wasn't found, try to see if "host" is the altname
          *   for this node, then lookup with the real name
          */
-        if (rc < 0 && (genders_errnum(gh) == GENDERS_ERR_NOTFOUND)) 
+        if (rc < 0 && (genders_errnum(gh) == GENDERS_ERR_NOTFOUND))
             rc = attrval_by_altname (gh, host, rcmd_attr, val, sizeof (val));
-        
+
         rcmd_type_parse (val, &rcmd, &user);
 
-        if (rc > 0) 
+        if (rc > 0)
             rcmd_register_defaults (host, rcmd, user);
 
         free (host);
     }
 
     hostlist_iterator_destroy (i);
-            
+
     return 0;
 }
 
-static int 
+static int
 _delete_all (hostlist_t hl, hostlist_t dl)
 {
     int                 rc   = 0;
