@@ -1,24 +1,24 @@
 /*****************************************************************************\
- *  $Id: testcase.c 1125 2007-06-04 17:27:56Z grondo $
+ *  $Id$
  *****************************************************************************
  *  Copyright (C) 2001-2002 The Regents of the University of California.
  *  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
  *  Written by Jim Garlick <garlick@llnl.gov>.
  *  UCRL-CODE-2003-005.
- *  
+ *
  *  This file is part of Pdsh, a parallel remote shell program.
  *  For details, see <http://www.llnl.gov/linux/pdsh/>.
- *  
+ *
  *  Pdsh is free software; you can redistribute it and/or modify it under
  *  the terms of the GNU General Public License as published by the Free
  *  Software Foundation; either version 2 of the License, or (at your option)
  *  any later version.
- *  
+ *
  *  Pdsh is distributed in the hope that it will be useful, but WITHOUT ANY
  *  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  *  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
  *  details.
- *  
+ *
  *  You should have received a copy of the GNU General Public License along
  *  with Pdsh; if not, write to the Free Software Foundation, Inc.,
  *  59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
@@ -70,20 +70,19 @@ static void _testmsg(int testnum, testresult_t result)
 
 static testresult_t _test_xstrerrorcat(void)
 {
+    int e;
     testresult_t result = PASS;
 
-    for (errno = 1; errno < 100; errno++) {
-#if	HAVE_STRERROR
-        char *s2 = strerror(errno);
-#else
-        extern char *sys_errlist[];
-        char *s2 = sys_errlist[errno];
-#endif
+    for (e = 1; e < 100; e++) {
         char *s1 = NULL;
-
+        char *s2 = strerror(e);
+        errno = e;
         xstrerrorcat(&s1);
-        if (strcmp(s1, s2) != 0)
+        if (strcmp(s1, s2) != 0) {
+            err ("xsterrorcat (errno=%d) = \"%s\" (should be \"%s\")\n", e, s1, s2);
+
             result = FAIL;
+        }
         Free((void **) &s1);
     }
     return result;
