@@ -5,20 +5,20 @@
  *  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
  *  Written by Jim Garlick <garlick@llnl.gov>.
  *  UCRL-CODE-2003-005.
- *  
+ *
  *  This file is part of Pdsh, a parallel remote shell program.
  *  For details, see <http://www.llnl.gov/linux/pdsh/>.
- *  
+ *
  *  Pdsh is free software; you can redistribute it and/or modify it under
  *  the terms of the GNU General Public License as published by the Free
  *  Software Foundation; either version 2 of the License, or (at your option)
  *  any later version.
- *  
+ *
  *  Pdsh is distributed in the hope that it will be useful, but WITHOUT ANY
  *  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  *  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
  *  details.
- *  
+ *
  *  You should have received a copy of the GNU General Public License along
  *  with Pdsh; if not, write to the Free Software Foundation, Inc.,
  *  59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
@@ -187,10 +187,10 @@ static char * wcoll_ctx_resolve_path (struct wcoll_ctx *ctx,
         const char *file, char *buf, int len)
 {
     if (file[0] == '/')
-        strncpy (buf, file, len);
+        strncpy (buf, file, len - 1);
     else if ( file[0] == '.'
             && (file[1] == '/' || (file[1] == '.' && file[2] == '/')))
-       strncpy (buf, file, len);
+       strncpy (buf, file, len - 1);
     else {
         if (wcoll_ctx_path_lookup (ctx, file, buf, len) < 0)
             return NULL;
@@ -317,9 +317,9 @@ static char * get_file_path (const char *file, char *dir, int len)
 
 /*
  * Read wcoll from specified file or from the specified FILE pointer.
- * (one of the arguments must be NULL).  
+ * (one of the arguments must be NULL).
  *      file (IN)       name of wcoll file (or NULL)
- *      f (IN)          FILE pointer to wcoll file (or NULL)    
+ *      f (IN)          FILE pointer to wcoll file (or NULL)
  *      RETURN          new list containing hostnames
  */
 hostlist_t read_wcoll(char *file, FILE * f)
